@@ -1,7 +1,7 @@
   /********************************************************************\
   * BitlBee -- An IRC to other IM-networks gateway                     *
   *                                                                    *
-  * Copyright 2002-2003 Wilmer van der Gaast and others                *
+  * Copyright 2002-2004 Wilmer van der Gaast and others                *
   \********************************************************************/
 
 /* Main file                                                            */
@@ -26,12 +26,17 @@
 #ifndef _BITLBEE_H
 #define _BITLBEE_H
 
-#define BITLBEE_VERSION "0.83"
+#define _GNU_SOURCE /* Stupid GNU :-P */
+
+#define PACKAGE "BitlBee"
+#define BITLBEE_VERSION "0.85"
+#define VERSION BITLBEE_VERSION
 
 #include "config.h"
 
 #include <fcntl.h>
 #include <unistd.h>
+#include <time.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <sys/time.h>
@@ -43,12 +48,15 @@
 #include <string.h>
 #include <stdarg.h>
 #include <stdio.h>
-#include <glib.h>
+#include <syslog.h>
+#include <errno.h>
 
 #ifndef NO_TCPD
 #include <tcpd.h>
 #endif
 
+#include <glib.h>
+
 #define _( x ) x
 
 #define ROOT_NICK "root"
@@ -64,8 +72,9 @@
 #define MAX_NICK_LENGTH 24
 
 #define HELP_FILE DATADIR "/help.txt"
-#define CONF_FILE ETCDIR "/bitlbee.conf"
-#define MOTD_FILE ETCDIR "/motd.txt"
+#define CONF_FILE_DEF ETCDIR "/bitlbee.conf"
+
+char *CONF_FILE;
 
 #include "irc.h"
 #include "set.h"
@@ -73,17 +82,39 @@
 #include "commands.h"
 #include "account.h"
 #include "conf.h"
+#include "log.h"
+#include "ini.h"
+#include "help.h"
+
+typedef struct global_t {
+	fd_set readfds[1];
+	fd_set writefds[1];
+	int listen_socket;
+	help_t *help;
+	conf_t *conf;
+} global_t;
+
+int bitlbee_daemon_main_loop( void );
+int bitlbee_inetd_main_loop( void );
+int bitlbee_daemon_init( void );
+int bitlbee_inetd_init( void );
+
+int bitlbee_connection_create( int fd ); 
+GList *bitlbee_connection_destroy( GList *connection );
 
 int root_command_string( irc_t *irc, user_t *u, char *command );
 int root_command( irc_t *irc, char *command[] );
-int bitlbee_init( irc_t *irc );
 int bitlbee_load( irc_t *irc, char *password );
 int bitlbee_save( irc_t *irc );
+double gettime( void );
 void http_encode( char *s );
 void http_decode( char *s );
-double gettime();
+
+void *bitlbee_alloc(size_t size);
+void *bitlbee_realloc(void *oldmem, size_t newsize);
 
 extern irc_t *IRC;
-extern conf_t *conf;
+extern global_t global;
+extern GList *connection_list;
 
 #endif
