@@ -1,12 +1,10 @@
   /********************************************************************\
   * BitlBee -- An IRC to other IM-networks gateway                     *
   *                                                                    *
-  * Copyright 2002-2003 Wilmer van der Gaast and others                *
+  * Copyright 2002-2004 Sjoerd Hemminga and others                     *
   \********************************************************************/
 
-/* A little bit of encryption for the users' passwords
-
-   Copyright 2002-2003 Sjoerd Hemminga                                  */
+/* A little bit of encryption for the users' passwords                  */
 
 /*
   This program is free software; you can redistribute it and/or modify
@@ -112,7 +110,7 @@ char *hashpass (irc_t *irc) {
 	
 	if (irc->password == NULL) return (NULL);
 	
-	rv = (char *)malloc (33);
+	rv = (char *)bitlbee_alloc (33);
 	memset (rv, 0, 33);
 	
 	md5_init (&md5state);
@@ -121,7 +119,7 @@ char *hashpass (irc_t *irc) {
 	
 	for (i = 0; i < 16; i++) {
 		/* Build a hash of the pass */
-		snprintf (digits, sizeof (digits), "%02x\n", digest[i]);
+		snprintf (digits, sizeof (digits), "%02x", digest[i]);
 		strcat (rv, digits);
 	}
 	
@@ -134,7 +132,7 @@ char *obfucrypt (irc_t *irc, char *line) {
 	
 	if (irc->password == NULL) return (NULL);
 	
-	rv = (char *)malloc (strlen (line) + 1);
+	rv = (char *)bitlbee_alloc (strlen (line) + 1);
 	memset (rv, '\0', strlen (line) + 1);
 	
 	i = j = 0;
@@ -159,7 +157,7 @@ char *deobfucrypt (irc_t *irc, char *line) {
 	
 	if (irc->password == NULL) return (NULL);
 	
-	rv = (char *)malloc (strlen (line) + 1);
+	rv = (char *)bitlbee_alloc (strlen (line) + 1);
 	memset (rv, '\0', strlen (line) + 1);
 	
 	i = j = 0;
