@@ -1,7 +1,7 @@
   /********************************************************************\
   * BitlBee -- An IRC to other IM-networks gateway                     *
   *                                                                    *
-  * Copyright 2002-2003 Wilmer van der Gaast and others                *
+  * Copyright 2002-2004 Wilmer van der Gaast and others                *
   \********************************************************************/
 
 /* INI file reading code						*/
@@ -23,6 +23,9 @@
   Suite 330, Boston, MA  02111-1307  USA
 */
 
+#ifndef _INI_H
+#define _INI_H
+
 #ifndef MAX_STRING
 #define MAX_STRING 128
 #endif
@@ -40,3 +43,5 @@ typedef struct
 ini_t *ini_open( char *file );
 int ini_read( ini_t *file );
 void ini_close( ini_t *file );
+
+#endif
