@@ -1,7 +1,7 @@
   /********************************************************************\
   * BitlBee -- An IRC to other IM-networks gateway                     *
   *                                                                    *
-  * Copyright 2002-2003 Wilmer van der Gaast and others                *
+  * Copyright 2002-2004 Wilmer van der Gaast and others                *
   \********************************************************************/
 
 /* Some stuff to fetch, save and handle nicknames for your buddies      */
@@ -43,9 +43,9 @@ void nick_set( irc_t *irc, char *handle, int proto, char *nick )
 	}
 	
 	if( m )
-		n = m->next = malloc( sizeof( nick_t ) );
+		n = m->next = bitlbee_alloc( sizeof( nick_t ) );
 	else
-		n = irc->nicks = malloc( sizeof( nick_t ) );
+		n = irc->nicks = bitlbee_alloc( sizeof( nick_t ) );
 	memset( n, 0, sizeof( nick_t ) );
 	
 	n->handle = strdup( handle );
