@@ -1,7 +1,7 @@
   /********************************************************************\
   * BitlBee -- An IRC to other IM-networks gateway                     *
   *                                                                    *
-  * Copyright 2002-2003 Wilmer van der Gaast and others                *
+  * Copyright 2002-2004 Wilmer van der Gaast and others                *
   \********************************************************************/
 
 /*
@@ -9,14 +9,12 @@
  *
  * Gaim without gaim - for BitlBee
  *
- * This file contains functions called by the Gaim IM-modules. It's written
- * from scratch for BitlBee and doesn't contain any code from Gaim anymore
- * (except for the function names).
- *
- * This include file contains some struct and type definitions from Gaim.
+ * This file contains functions called by the Gaim IM-modules. It contains
+ * some struct and type definitions from Gaim.
  *
  * Copyright (C) 1998-1999, Mark Spencer <markster@marko.net>
- * Copyright 2002-2003 Wilmer van der Gaast <lintux@lintux.cx>
+ *                          (and possibly other members of the Gaim team)
+ * Copyright 2002-2004 Wilmer van der Gaast <lintux@lintux.cx>
  */
 
 /*
@@ -129,6 +127,7 @@ struct conversation {
         char *channel;
         char *title;
         char joined;
+        void *data;
 };
 
 struct buddy {
@@ -202,6 +201,7 @@ struct prpl {
 	void (* chat_leave)	(struct gaim_connection *, int id);
 	void (* chat_whisper)	(struct gaim_connection *, int id, char *who, char *message);
 	int  (* chat_send)	(struct gaim_connection *, int id, char *message);
+	int  (* chat_open)	(struct gaim_connection *, char *who);
 	void (* keepalive)	(struct gaim_connection *);
 
 	/* get "chat buddy" info and away message */
@@ -216,11 +216,7 @@ struct prpl {
 
 	void (* buddy_free)	(struct buddy *);
 
-	/* this is really bad. */
-	void (* convo_closed)   (struct gaim_connection *, char *who);
-	int  (* convo_open)     (struct gaim_connection *, char *who);
-
-	char *(* normalize)(const char *);
+	char *(* get_status_string) (int stat);
 };
 
 #define PROTO_TOC	0
@@ -273,6 +269,7 @@ char *set_eval_charset(irc_t *irc, set_t *set, char *value);
 void nogaim_init();
 struct gaim_connection *gc_nr( int i );
 int proto_away( struct gaim_connection *gc, char *away );
+char *set_eval_away_devoice( irc_t *irc, set_t *set, char *value );
 int handle_cmp( char *a, char *b, int protocol );
 
 gboolean auto_reconnect( gpointer data );
@@ -347,7 +344,7 @@ void jabber_init( struct prpl *ret );
 
 #ifdef WITH_YAHOO
 /* yahoo.c */
-void yahoo_init( struct prpl *ret );
+void byahoo_init( struct prpl *ret );
 #endif
 
 /* prefs.c */
