@@ -1,7 +1,7 @@
   /********************************************************************\
   * BitlBee -- An IRC to other IM-networks gateway                     *
   *                                                                    *
-  * Copyright 2002-2003 Wilmer van der Gaast and others                *
+  * Copyright 2002-2004 Wilmer van der Gaast and others                *
   \********************************************************************/
 
 /* Some stuff to register, handle and save user preferences             */
@@ -176,3 +176,46 @@ char *set_eval_bool( irc_t *irc, set_t *set, char *value )
 		return( value );
 	return( set_eval_int( irc, set, value ) );
 }
+
+char *set_eval_to_char( irc_t *irc, set_t *set, char *value )
+{
+	char *s = bitlbee_alloc( 3 );
+	
+	if( *value == ' ' )
+		strcpy( s, " " );
+	else
+		sprintf( s, "%c ", *value );
+	
+	return( s );
+}
+
+char *set_eval_ops( irc_t *irc, set_t *set, char *value )
+{
+	if( strcasecmp( value, "user" ) == 0 )
+	{
+		irc_write( irc, ":%s!%s@%s MODE %s %s %s %s", irc->mynick, irc->mynick, irc->myhost,
+		                                              irc->channel, "+o-o", irc->nick, irc->mynick );
+		return( value );
+	}
+	else if( strcasecmp( value, "root" ) == 0 )
+	{
+		irc_write( irc, ":%s!%s@%s MODE %s %s %s %s", irc->mynick, irc->mynick, irc->myhost,
+		                                              irc->channel, "-o+o", irc->nick, irc->mynick );
+		return( value );
+	}
+	else if( strcasecmp( value, "both" ) == 0 )
+	{
+		irc_write( irc, ":%s!%s@%s MODE %s %s %s %s", irc->mynick, irc->mynick, irc->myhost,
+		                                              irc->channel, "+oo", irc->nick, irc->mynick );
+		return( value );
+	}
+	else if( strcasecmp( value, "none" ) == 0 )
+	{
+		irc_write( irc, ":%s!%s@%s MODE %s %s %s %s", irc->mynick, irc->mynick, irc->myhost,
+		                                              irc->channel, "-oo", irc->nick, irc->mynick );
+		return( value );
+	}
+	
+	return( NULL );
+}
+
