@@ -1,7 +1,7 @@
   /********************************************************************\
   * BitlBee -- An IRC to other IM-networks gateway                     *
   *                                                                    *
-  * Copyright 2002-2003 Wilmer van der Gaast and others                *
+  * Copyright 2002-2004 Wilmer van der Gaast and others                *
   \********************************************************************/
 
 /* Some stuff to register, handle and save user preferences             */
@@ -43,5 +43,10 @@ int set_getint( irc_t *irc, char *key );
 int set_setstr( irc_t *irc, char *key, char *value );
 int set_setint( irc_t *irc, char *key, int value );
 void set_del( irc_t *irc, char *key );
+
 char *set_eval_int( irc_t *irc, set_t *set, char *value );
 char *set_eval_bool( irc_t *irc, set_t *set, char *value );
+char *set_eval_to_char( irc_t *irc, set_t *set, char *value );
+char *set_eval_ops( irc_t *irc, set_t *set, char *value );
+
+
