@@ -1,7 +1,7 @@
   /********************************************************************\
   * BitlBee -- An IRC to other IM-networks gateway                     *
   *                                                                    *
-  * Copyright 2002-2003 Wilmer van der Gaast and others                *
+  * Copyright 2002-2004 Wilmer van der Gaast and others                *
   \********************************************************************/
 
 /* Stuff to handle, save and search buddies                             */
@@ -38,6 +38,10 @@ typedef struct user
 	char *handle;
 	struct gaim_connection *gc;
 	
+	char *sendbuf;
+	int sendbuf_len;
+	guint sendbuf_timer;
+	
 	int (*send_handler) ( irc_t *irc, struct user *u, char *msg );
 	
 	void *next;
